/*
 * Copyright (c) 2011 Sven Killig
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.killig.liveview;

import java.util.ArrayList;

import android.app.PendingIntent;
import android.app.PendingIntent.CanceledException;
import android.content.ComponentName;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;

import com.sonyericsson.extras.liveview.plugins.AbstractPluginService;
import com.sonyericsson.extras.liveview.plugins.PluginConstants;

class LiveViewMessage {
  String imageAnnounce, header, body, openInPhoneAction;
  long timestamp;
  
  public LiveViewMessage(String imageAnnounce, String header, String body, long timestamp, String openInPhoneAction) {
  	this.imageAnnounce=imageAnnounce;
  	this.header=header;
  	this.body=body;
  	this.timestamp=timestamp;
  	this.openInPhoneAction=openInPhoneAction;
  }
}

public class BroadcastAnnounceGatewayService extends AbstractPluginService {
  private static final String LOG_TAG = "BroadcastAnnounceGatewayService";
    
  // Our handler.
  private Handler mHandler = null;
  
  // Is loop running?
  private boolean mWorkerRunning = false;
  
  ArrayList<LiveViewMessage> messageQueue=new ArrayList<LiveViewMessage>(), messageQueue2=new ArrayList<LiveViewMessage>();
    
	@Override
	public void onStart(Intent intent, int startId) {
		super.onStart(intent, startId);
		Log.d(LOG_TAG, "onStart(");

    // Create handler.
    if(mHandler == null) {
        mHandler = new Handler();
    }
    
    String header=null;
    if(intent!=null) header=intent.getStringExtra("header");
    Log.d(LOG_TAG, "############ "+header);
    if(header!=null) sendAnnounce(intent.getStringExtra("imageAnnounce"), header, intent.getStringExtra("body"), intent.getLongExtra("timestamp",System.currentTimeMillis()), intent.getStringExtra("openInPhoneAction"));
	}
	
	/*@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
	int ret=super.onStartCommand(intent, flags, startId);
	Log.d(LOG_TAG, "2############"+intent.getStringExtra(Intent.EXTRA_TEXT));
	return ret;
	}*/
	
	@Override
	public void onCreate() {
		super.onCreate();
		Log.d(LOG_TAG, "onCreate()");
		// Do plugin specifics.
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		Log.d(LOG_TAG, "onDestroy()");
		// Do plugin specifics.
	}
	
  /**
   * Plugin is just sending notifications.
   */
  protected boolean isSandboxPlugin() {
      return false;
  }
	
	/**
	 * Must be implemented. Starts plugin work, if any.
	 */
	protected void startWork() {
		Log.d(LOG_TAG, "startWork()");
		// Check if plugin is enabled.
		if(!mWorkerRunning && mSharedPreferences.getBoolean(PluginConstants.PREFERENCES_PLUGIN_ENABLED, false)) {
		    mWorkerRunning = true;
//			scheduleTimer();
//	    mLiveViewAdapter.sendAnnounce(mPluginId, mMenuIcon, title, text, System.currentTimeMillis(), null);
		}
		synchronized(messageQueue) {
			for(LiveViewMessage lvm : messageQueue) {
				sendAnnounce(lvm.imageAnnounce, lvm.header, lvm.body, lvm.timestamp, lvm.openInPhoneAction);
			}
			messageQueue.removeAll(messageQueue);
			synchronized(messageQueue2) {
				messageQueue.addAll(messageQueue2);
				messageQueue2.removeAll(messageQueue2);
			}
		}
	}
	
	/**
	 * Must be implemented. Stops plugin work, if any.
	 */
	protected void stopWork() {
		Log.d(LOG_TAG, "stopWork()");
	    mWorkerRunning = false;
	}
	
	/**
	 * Must be implemented.
	 * 
	 * PluginService has done connection and registering to the LiveView Service. 
	 * 
	 * If needed, do additional actions here, e.g. 
	 * starting any worker that is needed.
	 */
	protected void onServiceConnectedExtended(ComponentName className, IBinder service) {}
	
	/**
	 * Must be implemented.
	 * 
	 * PluginService has done disconnection from LiveView and service has been stopped. 
	 * 
	 * Do any additional actions here.
	 */
	protected void onServiceDisconnectedExtended(ComponentName className) {}

	/**
	 * Must be implemented.
	 * 
	 * PluginService has checked if plugin has been enabled/disabled.
	 * 
	 * The shared preferences has been changed. Take actions needed. 
	 */	
	protected void onSharedPreferenceChangedExtended(SharedPreferences pref, String key) {}

	protected void startPlugin() {
		Log.d(LOG_TAG, "startPlugin()");
		startWork();
	}
			
	protected void stopPlugin() {
		Log.d(LOG_TAG, "stopPlugin()");
		stopWork();
	}

	protected void button(String buttonType, boolean doublepress, boolean longpress) {
        Log.d(LOG_TAG, "button - type " + buttonType + ", doublepress " + doublepress + ", longpress " + longpress);
    }

	protected void displayCaps(int displayWidthPx, int displayHeigthPx) {
        Log.d(LOG_TAG, "displayCaps - width " + displayWidthPx + ", height " + displayHeigthPx);
    }

	protected void onUnregistered() {
		Log.d(LOG_TAG, "onUnregistered()");
		stopWork();
	}

	protected void openInPhone(String openInPhoneAction) {
		Log.d(LOG_TAG, "openInPhone: " + openInPhoneAction);
		if(openInPhoneAction!=null) {
			if(openInPhoneAction.endsWith("/")) {
				String packageName = openInPhoneAction.substring(0, openInPhoneAction.length()-1);
				Log.d(LOG_TAG, "packageName: " + packageName);
				startActivity(getPackageManager().getLaunchIntentForPackage(packageName));
			} else if(openInPhoneAction.endsWith("#")) {
				int hashCode=Integer.parseInt(openInPhoneAction.substring(0, openInPhoneAction.length()-1));
				PendingIntent pi=NotificationListener.hm.get(hashCode);
				try {
					if(pi!=null) pi.send();
				} catch (CanceledException e) {
					e.printStackTrace();
				}
			} else {
		    try {
		      Process sh;
		      sh = Runtime.getRuntime().exec(openInPhoneAction);
		      if ((sh.waitFor() != 0)) {
						Log.e(LOG_TAG, "openInPhone failed!");
		      }
				} catch (Exception e) {
		      e.printStackTrace();
				}
			}
		}
	}
	
	protected void screenMode(int mode) {
	    Log.d(LOG_TAG, "screenMode: screen is now " + ((mode == 0) ? "OFF" : "ON"));
	}
			
	private void sendAnnounce(String imageAnnounce, String header, String body, long timestamp, String openInPhoneAction) {
		Log.d(LOG_TAG, "sendAnnounce(");
		try {
			if(mWorkerRunning && (mLiveViewAdapter != null) && mSharedPreferences.getBoolean(PluginConstants.PREFERENCES_PLUGIN_ENABLED, false)) {
		    mLiveViewAdapter.sendAnnounce(mPluginId, imageAnnounce, header, body, timestamp, openInPhoneAction);
				Log.d(LOG_TAG, "Announce sent to LiveView");
			} else {
				synchronized(messageQueue2) {
					messageQueue2.add(new LiveViewMessage(imageAnnounce, header, body, timestamp, openInPhoneAction));
				}
				Log.d(LOG_TAG, "LiveView not reachable");
			}
		} catch(Exception e) {
			Log.e(LOG_TAG, "Failed to send announce", e);
		}
	}

}