package de.killig.liveview;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.Map;

import android.annotation.TargetApi;
import android.app.Notification;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.service.notification.NotificationListenerService;
import android.service.notification.StatusBarNotification;
import android.util.Log;

import com.sonyericsson.extras.liveview.plugins.PluginConstants;
import com.sonyericsson.extras.liveview.plugins.PluginUtils;

@TargetApi(19)
public class NotificationListener extends NotificationListenerService {
  private static final String LOG_TAG = "NotificationListener";
  public static Map<Integer,PendingIntent> hm=new HashMap<Integer,PendingIntent>();

	@Override
	public void onNotificationPosted(StatusBarNotification sbn) {
		if(!PreferenceManager.getDefaultSharedPreferences(this).getBoolean(PluginConstants.PREFERENCES_PLUGIN_ENABLED, false)) return;
		
		Log.d(LOG_TAG, "sbn:"+sbn.toString());
		
		Notification n=sbn.getNotification();
		Log.d(LOG_TAG, "n:"+n);
		// https://github.com/tedk/WebkitNotifications/blob/master/android/src/net/homeip/tedk/webkitnotifications/NotificationListenerService.java#L161
		String header = "Notification";
		String body = n.tickerText == null || n.tickerText.toString().trim().length() == 0 ? null : n.tickerText.toString().trim();
		Log.d(LOG_TAG, "tickerText="+body);
		Log.d(LOG_TAG, "contentIntent="+n.contentIntent);
		if(body==null) {
			if(Build.VERSION.SDK_INT>=19) {
				Bundle b=n.extras;
				header=b.getString(Notification.EXTRA_TITLE);
				Log.d(LOG_TAG, "EXTRA_TITLE="+header);
				body=b.getString(Notification.EXTRA_TEXT);
				Log.d(LOG_TAG, "EXTRA_TEXT="+body);
			}
		}
		boolean vibrate = (n.vibrate != null && n.vibrate.length > 0) || ((n.defaults & Notification.DEFAULT_VIBRATE) == Notification.DEFAULT_VIBRATE);
		boolean lights = ((n.flags & Notification.FLAG_SHOW_LIGHTS) == Notification.FLAG_SHOW_LIGHTS) || ((n.defaults & Notification.DEFAULT_LIGHTS) == Notification.DEFAULT_LIGHTS);
		Log.d(LOG_TAG, "vibrate="+vibrate+"; lights="+lights);
		
		if(body!=null && (vibrate || lights)) {
			Context context=getApplicationContext();
			String serviceIntent = PluginUtils.getDynamicResourceString(context, PluginConstants.RESOURCE_STRING_INTENT_SERVICE);
			Intent i=new Intent(serviceIntent);
			
			

			// https://github.com/tedk/WebkitNotifications/blob/master/android/src/net/homeip/tedk/webkitnotifications/NotificationListenerService.java#L191
			PackageManager pm = context.getPackageManager();
					
			String packageName = sbn.getPackageName().toString();
			Log.d(LOG_TAG, "packageName="+packageName);
			if(NotificationReceiver.K9_PACKAGE.equals(packageName) || NotificationReceiver.KAITEN_PACKAGE.equals(packageName)) return;
			String appName = null;
			try {
			    appName = pm.getApplicationLabel(pm.getApplicationInfo(packageName, 0)).toString();
			} catch (Exception e) {
			    Log.e(LOG_TAG, "Could not load application name", e);
			}
			if (appName == null) appName = packageName;
			Context c = getApplicationContext(); //!
			try {
			    c = createPackageContext(packageName, 0); //!
			} catch (NameNotFoundException e) {
			    Log.e(LOG_TAG, "Could not load application context", e);
			}
/*			String icon = null;
			ByteArrayOutputStream baos = null;
			try {
			    baos = new ByteArrayOutputStream();
			    BitmapFactory.Options options = new BitmapFactory.Options();
			    options.inPreferredConfig = Bitmap.Config.ARGB_8888;
			    Bitmap iconBitmap = BitmapFactory.decodeResource(c.getResources(), n.icon, options);
			    Bitmap iconWithBackground = Bitmap.createBitmap(
				    iconBitmap.getWidth(), iconBitmap.getHeight(),
				    iconBitmap.getConfig());
			    Canvas background = new Canvas(iconWithBackground);
			    background.drawColor(Color.BLACK);
			    background.drawBitmap(iconBitmap, 0.f, 0.f, null);
			    iconWithBackground.compress(Bitmap.CompressFormat.PNG, 100, baos);
			    baos.flush();
			    icon = "data:image/png;base64,"
				    + Base64.encodeToString(baos.toByteArray(), Base64.NO_WRAP
					    | Base64.URL_SAFE);
			} catch (Exception e) {
			    Log.e("NotificationListenerService", "Could not load icon", e);
			} finally {
			    if (baos != null)
				try {
				    baos.close();
				} catch (Exception e) {
				}
			}*/
			
			
			File path=new File(Environment.getExternalStorageDirectory().getPath()+"/Android/data/de.killig.liveview/files");
			String name=packageName+".png";
			File f=new File(path, name);
			FileOutputStream fos=null;
			try {
				fos = new FileOutputStream(f);
/*		    BitmapFactory.Options options = new BitmapFactory.Options();
		    options.inPreferredConfig = Bitmap.Config.ARGB_8888;*/
		    Bitmap iconBitmap = BitmapFactory.decodeResource(c.getResources(), n.icon, /*options*/null);
		    iconBitmap.compress(CompressFormat.PNG, 0, fos);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}

			
			i.putExtra("imageAnnounce", f.getPath());
			i.putExtra("header", header);
			i.putExtra("body", body);
			i.putExtra("timestamp", n.when);
			PendingIntent pi=n.contentIntent;
			if(pi!=null) {
				hm.put(pi.hashCode(), pi);
				i.putExtra("openInPhoneAction", pi.hashCode()+"#");
			} else {
				i.putExtra("openInPhoneAction", packageName+"/");
			}
			context.startService(i);
		}
	}

	
	@Override
	public void onNotificationRemoved(StatusBarNotification sbn) {
		if(!PreferenceManager.getDefaultSharedPreferences(this).getBoolean(PluginConstants.PREFERENCES_PLUGIN_ENABLED, false)) return;
		hm.remove(sbn.getNotification().contentIntent.hashCode());
	}

}
