/*
 * Copyright (c) 2011 Sven Killig
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.killig.liveview;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.Date;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.res.AssetFileDescriptor;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.util.Log;

//import com.fsck.k9.EmailReceivedIntent;
import com.sonyericsson.extras.liveview.plugins.PluginConstants;
import com.sonyericsson.extras.liveview.plugins.PluginUtils;

/**
 * Receives broadcast intents from LiveView service.
 */
public class NotificationReceiver extends BroadcastReceiver {
  private static final String LOG_TAG = "NotificationReceiver";
  static final String K9_PACKAGE     = "com.fsck.k9";
  private static final String K9     = K9_PACKAGE+".intent.";
  static final String KAITEN_PACKAGE = "com.kaitenmail";
  private static final String KAITEN = KAITEN_PACKAGE+".intent.";
	
  @Override
	public void onReceive(Context context, Intent intent) {
		if(!PreferenceManager.getDefaultSharedPreferences(context).getBoolean(PluginConstants.PREFERENCES_PLUGIN_ENABLED, false)) return;

		Log.d(LOG_TAG, "Transferring Intent: " + intent);
		String serviceIntent = PluginUtils.getDynamicResourceString(context, PluginConstants.RESOURCE_STRING_INTENT_SERVICE);
		Intent i=new Intent(serviceIntent);
		String action=intent.getAction(), prefix=null;
		if(action!=null) {
			if(action.startsWith(K9)) prefix=K9;
			else if(action.startsWith(KAITEN)) prefix=KAITEN;
		}
//		if(EmailReceivedIntent.ACTION_EMAIL_RECEIVED.equals(intent.getAction())) {
		if(prefix!=null) {
			prefix+="extra.";
			File path=new File(Environment.getExternalStorageDirectory().getPath()+"/Android/data/de.killig.liveview/files");
			String name="stat_notify_email_generic.png";
			File f=new File(path, name);
			if(!f.isFile()) {
				FileChannel inChannel=null, outChannel=null;
				try {
					path.mkdirs();
					// strange FileNotFoundException when using res/raw:
					//inChannel = new FileInputStream(context.getResources().openRawResourceFd(R.raw.stat_notify_email_generic).getFileDescriptor()).getChannel();
					AssetFileDescriptor afd=context.getAssets().openFd(name);
					inChannel = new FileInputStream(afd.getFileDescriptor()).getChannel();
					outChannel = new FileOutputStream(f).getChannel();
          inChannel.transferTo(afd.getStartOffset(), afd.getLength(), outChannel);
				} catch (IOException e) {
          e.printStackTrace();
				} finally {
						try {
							if (inChannel != null) inChannel.close();
		          if (outChannel != null) outChannel.close();
						} catch (IOException e) {
							e.printStackTrace();
						}
				}
			}
			Log.d(LOG_TAG, "imageAnnounce: "+f.getPath());
      i.putExtra("imageAnnounce", f.getPath());
			i.putExtra("header", intent.getStringExtra(prefix+"SUBJECT"));
			i.putExtra("body",""
					+"From: "+intent.getStringExtra(prefix+"FROM")+"; "
			    +"Account: "+intent.getStringExtra(prefix+"ACCOUNT")+"; "
			    +"Folder: "+intent.getStringExtra(prefix+"FOLDER")+"; "
			    +"To: "+intent.getStringExtra(prefix+"TO")/*+"; "
			    +"Cc: "+intent.getStringExtra(EmailReceivedIntent.EXTRA_CC)*/
			);
			if(intent.hasExtra(prefix+"SENT_DATE")) i.putExtra("timestamp", ((Date)intent.getSerializableExtra(prefix+"SENT_DATE")).getTime());
      i.putExtra("openInPhoneAction", "am start -a "+Intent.ACTION_VIEW+" -d "+intent.getDataString());
		} else {
			i.putExtra("imageAnnounce", intent.getStringExtra("imageAnnounce"));
			i.putExtra("header", intent.getStringExtra("header"));
			i.putExtra("body", intent.getStringExtra("body"));
			if(intent.hasExtra("timestamp")) i.putExtra("timestamp", intent.getLongExtra("timestamp", System.currentTimeMillis()));
			i.putExtra("openInPhoneAction", intent.getStringExtra("openInPhoneAction"));
		}
		context.startService(i);
	}
}